@@ -1,3 +1,4 @@
+// Unicode support by Jim Park -- 08/22/2007
 // For layered windows
 #define _WIN32_WINNT 0x0500
 
@@ -5,27 +6,27 @@
 #include <windowsx.h>
 #include <nsis/pluginapi.h> // nsis plugin
 
-HINSTANCE g_hInstance;
+#ifndef LWA_COLORKEY
+#  define LWA_COLORKEY 1
+#  define LWA_ALPHA 2
+#endif
+
 
 #define RESOLUTION 32 // 30 fps ;) (32? I like SHR more than iDIV ;)
 
-BITMAP bm;
+HINSTANCE g_hInstance;
 HBITMAP g_hbm;
+BITMAP bm;
 int g_rv;
 int resolution;
-int sleep_val, fadein_val, fadeout_val, state, timeleft, keycolor, nt50,
-    alphaparam;
-const char classname[4] = "_sp";
+int sleep_val, fadein_val, fadeout_val, state, timeleft, keycolor, alphaparam;
+const TCHAR classname[4] = _T("_sp");
 
-typedef BOOL(_stdcall * _tSetLayeredWindowAttributesProc) (HWND hwnd,      // handle to the layered window
-                                                           COLORREF crKey, // specifies the color key
-                                                           BYTE bAlpha,    // value for the blend function
-                                                           DWORD dwFlags   // action
-    );
-_tSetLayeredWindowAttributesProc SetLayeredWindowAttributesProc;
+typedef BOOL(WINAPI*SetLayeredWindowAttributes_T)(HWND hwnd, COLORREF crKey, BYTE bAlpha, DWORD dwFlags);
+SetLayeredWindowAttributes_T SetLayeredWindowAttributesProc;
+#define IsLayeredWnd() ( SetLayeredWindowAttributesProc != NULL )
 
-static LRESULT CALLBACK WndProc(HWND hwnd, UINT uMsg, WPARAM wParam,
-                                LPARAM lParam)
+static LRESULT CALLBACK WndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
   PAINTSTRUCT ps;
   RECT r;
@@ -36,7 +37,7 @@ static LRESULT CALLBACK WndProc(HWND hwnd, UINT uMsg, WPARAM wParam,
   switch (uMsg) {
   case WM_CREATE:
     SystemParametersInfo(SPI_GETWORKAREA, 0, &r, 0);
-    SetWindowLong(hwnd, GWL_STYLE, 0);
+    SetWindowLongPtr(hwnd, GWL_STYLE, 0);
     SetWindowPos(hwnd, NULL,
                  r.left + (r.right - r.left - bm.bmWidth) / 2,
                  r.top + (r.bottom - r.top - bm.bmHeight) / 2,
@@ -125,8 +126,7 @@ BOOL WINAPI DllMain(HINSTANCE hInst, ULONG ul_reason_for_call, LPVOID lpReserved
   return TRUE;
 }
 
-void CALLBACK TimeProc(UINT uID,
-                       UINT uMsg, DWORD dwUser, DWORD dw1, DWORD dw2)
+void CALLBACK TimeProc(UINT uID, UINT uMsg, DWORD_PTR dwUser, DWORD_PTR dw1, DWORD_PTR dw2)
 {
   int call = -1;
   switch (state) {
@@ -135,7 +135,7 @@ void CALLBACK TimeProc(UINT uID,
     if (timeleft == 0) {
       timeleft = sleep_val;
       state++;
-      if (nt50)
+      if (IsLayeredWnd())
         call = 255;
     } else {
       call = ((fadein_val - timeleft) * 255) / fadein_val;
@@ -160,20 +160,22 @@ void CALLBACK TimeProc(UINT uID,
     }
   }
   // Transparency value aquired, and could be set...
-  if ((call >= 0) && nt50)
-    SetLayeredWindowAttributesProc((HWND) dwUser, keycolor,
-                                   (BYTE) call, alphaparam);
+  if ((call >= 0) && IsLayeredWnd())
+  {
+    SetLayeredWindowAttributesProc((HWND) dwUser, keycolor, (BYTE) call, alphaparam);
+  }
 
   // Time is running out...
   timeleft--;
 }
 
 void __declspec(dllexport) show(HWND hwndParent, int string_size,
-                                char *variables, stack_t ** stacktop)
+                                TCHAR *variables, stack_t ** stacktop)
 {
   DEVMODE dm;
-  char fn[MAX_PATH];
-  char temp[64];
+  TCHAR fn[MAX_PATH], uselayerwnd;
+  TCHAR temp[64];
+  FARPROC slwa;
 
   g_rv = -1;
   resolution = RESOLUTION;
@@ -192,10 +194,16 @@ void __declspec(dllexport) show(HWND hwndParent, int string_size,
 
   dm.dmSize = sizeof(DEVMODE);
   EnumDisplaySettings(NULL, ENUM_CURRENT_SETTINGS, &dm);
+#ifdef _WIN64
+  slwa = (FARPROC) SetLayeredWindowAttributes;
+#else
+  slwa = GetProcAddress(LoadLibraryA("USER32"), "SetLayeredWindowAttributes");
+#endif
+
   // Check for winXP/2k at 32 bpp transparency
-  nt50 = (LOBYTE(LOWORD(GetVersion())) >= 5) && !((dm.dmBitsPerPel < 32)
-                                                  && (keycolor != -1));
-  if (!nt50) {
+  uselayerwnd = slwa && dm.dmBitsPerPel >= 32 && keycolor == -1;
+
+  if (!uselayerwnd) {
     // Fading+transparency is unsupported at old windows versions...
     resolution = sleep_val + fadein_val + fadeout_val;
     fadeout_val = fadein_val = 0;
@@ -222,33 +230,30 @@ void __declspec(dllexport) show(HWND hwndParent, int string_size,
     wc.hCursor = LoadCursor(NULL, IDC_ARROW);
     wc.lpszClassName = classname;
     if (RegisterClass(&wc)) {
-      char fn2[MAX_PATH];
-      lstrcpy(fn2, fn);
-      lstrcat(fn, ".bmp");
-      lstrcat(fn2, ".wav");
-      g_hbm =
-          LoadImage(NULL, fn, IMAGE_BITMAP, 0, 0,
-                    LR_CREATEDIBSECTION | LR_LOADFROMFILE);
+      TCHAR fn2[MAX_PATH];
+
+      fn2[0] = _T('\0'), lstrcat(fn2, fn);
+      lstrcat(fn2, _T(".wav"));
+      lstrcat(fn, _T(".bmp"));
+
+      g_hbm = LoadImage(NULL, fn, IMAGE_BITMAP, 0, 0,
+                LR_CREATEDIBSECTION | LR_LOADFROMFILE);
       if (g_hbm) {
         HWND myWnd;
         UINT timerEvent;
 
         // Get Bitmap Information
-        GetObject(g_hbm, sizeof(bm), (LPSTR) & bm);
+        GetObject(g_hbm, sizeof(bm), & bm);
 
-        myWnd =
-            CreateWindowEx(WS_EX_TOOLWINDOW |
-                           ((nt50) ? (WS_EX_LAYERED) : (0)), classname,
+        myWnd = CreateWindowEx(WS_EX_TOOLWINDOW |
+                           (uselayerwnd ? (WS_EX_LAYERED) : (0)), classname,
                            classname, 0, 0, 0, 0, 0, (HWND) hwndParent,
                            NULL, g_hInstance, NULL);
 
         // Set transparency / key color
-        if (nt50) {
-          // Get blending proc address
-          HANDLE user32 = GetModuleHandle("user32");
-          SetLayeredWindowAttributesProc =
-              (_tSetLayeredWindowAttributesProc) GetProcAddress(user32,
-                                                                "SetLayeredWindowAttributes");
+        if (uselayerwnd) {
+          SetLayeredWindowAttributesProc = (SetLayeredWindowAttributes_T) slwa;
+
           // Use win2k method
           SetLayeredWindowAttributesProc(myWnd, keycolor,
                                          (BYTE) ((fadein_val > 0) ? (0) : (255)),
@@ -279,9 +284,9 @@ void __declspec(dllexport) show(HWND hwndParent, int string_size,
         DeleteObject(g_hbm);
       }
       // We should UnRegister class, since Windows NT series never does this by itself
-      UnregisterClass(wc.lpszClassName, g_hInstance);
+      UnregisterClass(wc.lpszClassName, wc.hInstance);
     }
   }
-  wsprintf(temp, "%d", g_rv);
+  wsprintf(temp, _T("%d"), g_rv);
   pushstring(temp);
 }
