@@ -219,9 +219,9 @@ static void ToolBarSizeChanged(HWND hDlg)
   SetWindowPos(hEd, 0, pt.x, top, RectW(r), oldh + (pt.y - top), SWP_NOZORDER|SWP_NOACTIVATE); // Update IDC_LOGWIN position and size
 }
 
-static BOOL CALLBACK DialogResize(HWND hWnd, LPARAM /* unused */)
+static BOOL CALLBACK DialogResize(HWND hWnd, LPARAM param)
 {
-  RECT r, r2;
+  RECT r, r2, &dlgrect = *(RECT*) param;
   GetWindowRect(hWnd, &r);
   ScreenToClient(g_sdata.hwnd, ((LPPOINT)&r)+0), ScreenToClient(g_sdata.hwnd, ((LPPOINT)&r)+1);
   switch (GetDlgCtrlID(hWnd))
@@ -231,7 +231,13 @@ static BOOL CALLBACK DialogResize(HWND hWnd, LPARAM /* unused */)
     SetWindowPos(hWnd, 0, 0, 0, RectW(r) + g_resize.dx, RectH(r2), SWP_NOMOVE|SWP_NOZORDER|SWP_NOACTIVATE);
     break;
   case IDC_LOGWIN:
-    SetWindowPos(hWnd, 0, r.left, r.top, RectW(r) + g_resize.dx, RectH(r) + g_resize.dy, SWP_NOZORDER|SWP_NOMOVE|SWP_NOACTIVATE);
+    if (!g_resize.bottompanelsize)
+    {
+      r2.top = 246, r2.bottom = 22 + 190; // Dialog units from the .rc file
+      MapDialogRect(GetParent(hWnd), &r2);
+      g_resize.bottompanelsize = r2.top - r2.bottom;
+    }
+    SetWindowPos(hWnd, 0, r.left, r.top, dlgrect.right - (r.left * 2), dlgrect.bottom - (r.top + g_resize.bottompanelsize), SWP_NOZORDER|SWP_NOMOVE|SWP_NOACTIVATE);
     break;
   case IDC_TEST:
   case IDCANCEL:
@@ -353,7 +359,7 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
       GetClientRect(hwndDlg, &g_resize.resizeRect);
       g_resize.dx = g_resize.resizeRect.right - oldW;
       g_resize.dy = g_resize.resizeRect.bottom - oldH;
-      EnumChildWindows(g_sdata.hwnd, DialogResize, (LPARAM) 0);
+      EnumChildWindows(g_sdata.hwnd, DialogResize, (LPARAM) &g_resize.resizeRect);
       return TRUE;
     }
     case WM_SIZING:
@@ -462,18 +468,24 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
       }
       EnableItems(g_sdata.hwnd);
       if (!g_sdata.retcode) {
-        MessageBeep(MB_ICONASTERISK);
-        if (g_sdata.warnings)
+        if (g_sdata.warnings) {
           SetTitle(g_sdata.hwnd,_T("Finished with Warnings"));
-        else
+          PlayAppSoundAsync(("BuildWarning"), MB_ICONWARNING);
+          SetLogColor(LC_WARNING);
+        }
+        else {
           SetTitle(g_sdata.hwnd,_T("Finished Sucessfully"));
+          PlayAppSoundAsync(("BuildComplete"), MB_ICONASTERISK);
+          SetLogColor(LC_SUCCESS);
+        }
         // Added by Darren Owen (DrO) on 1/10/2003
         if(g_sdata.recompile_test)
           PostMessage(g_sdata.hwnd, WM_COMMAND, LOWORD(IDC_TEST), 0);
       }
       else {
-        MessageBeep(MB_ICONEXCLAMATION);
         SetTitle(g_sdata.hwnd,_T("Compile Error: See Log for Details"));
+        PlayAppSoundAsync(("BuildError"), MB_ICONEXCLAMATION);
+        SetLogColor(LC_ERROR);
       }
 
       // Added by Darren Owen (DrO) on 1/10/2003
@@ -485,9 +497,10 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
     case MakensisAPI::QUERYHOST: {
       if (MakensisAPI::QH_OUTPUTCHARSET == wParam) {
         const UINT reqcp = 1200; // We want UTF-16LE
-        SetWindowLongPtr(hwndDlg, DWLP_MSGRESULT, (LONG_PTR)(1+reqcp));
-        return TRUE;
+        return DlgRet(hwndDlg, (LONG_PTR)(1+reqcp));
       }
+      else if (MakensisAPI::QH_SUPPORTEDVERSION == wParam)
+        return DlgRet(hwndDlg, 0x03006000);
       return FALSE;
     }
     case WM_NOTIFY:
@@ -534,7 +547,8 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
       return TRUE;
     case WM_COPYDATA:
     {
-      PCOPYDATASTRUCT cds = PCOPYDATASTRUCT(lParam);
+      using namespace MakensisAPI;
+      COPYDATASTRUCT *cds = (COPYDATASTRUCT*) lParam, cdsret;
       switch (cds->dwData) {
         case MakensisAPI::NOTIFY_SCRIPT:
           MemSafeFree(g_sdata.input_script);
@@ -552,6 +566,24 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
           g_sdata.output_exe = (TCHAR*) MemAlloc(cds->cbData * sizeof(TCHAR));
           lstrcpy(g_sdata.output_exe, (TCHAR *)cds->lpData);
           break;
+        case MakensisAPI::PROMPT_FILEPATH:
+          if ((((PROMPT_FILEPATH_DATA*)cds->lpData)->Platform & 7) == sizeof(TCHAR))
+          {
+            TCHAR buf[MAX_PATH];
+            lstrcpyn(buf, FSPath::FindLastComponent(((PROMPT_FILEPATH_DATA*)cds->lpData)->Path), COUNTOF(buf));
+            OPENFILENAME of = { sizeof(of) };
+            of.hwndOwner = hwndDlg;
+            of.lpstrFilter = _T("*.exe\0*.exe\0*\0*.*\0");
+            of.lpstrFile = buf, of.nMaxFile = COUNTOF(buf);
+            of.Flags = OFN_EXPLORER|OFN_ENABLESIZING|OFN_HIDEREADONLY|OFN_OVERWRITEPROMPT|OFN_PATHMUSTEXIST|OFN_NOCHANGEDIR;
+            if (GetSaveFileName(&of))
+            {
+              cdsret.dwData = cds->dwData, cdsret.cbData = (lstrlen(buf) + 1) * sizeof(TCHAR), cdsret.lpData = buf;
+              SendMessage((HWND) wParam, WM_COPYDATA, (SIZE_T) hwndDlg, (SIZE_T) &cdsret);
+            }
+            return TRUE;
+          }
+          return FALSE;
       }
       return TRUE;
     }
@@ -569,6 +601,9 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
     case WM_COMMAND:
     {
       switch (LOWORD(wParam)) {
+        case IDM_UI_SWITCHSECTION: //devblogs.microsoft.com/oldnewthing/20191022-00/?p=103016
+          SetDialogFocus(hwndDlg, g_toolbar.hwnd); // Toolbar does not have WS_TABSTOP and we have no other "UI areas" to switch to so just go there
+          break;
         case IDM_BROWSESCR: {
           if (g_sdata.input_script) {
             TCHAR str[MAX_PATH],*str2;
@@ -655,6 +690,17 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
           extern int ShowWndSpy(HWND hOwner);
           return ShowWndSpy(g_sdata.hwnd);
         }
+        case IDM_GUIDGEN:
+        {
+          GUID guid;
+          TCHAR buf[41 * (1 + (sizeof(TCHAR) < 2))];
+          FARPROC func = GetKeyState(VK_CONTROL) < 0 ? GetSysProcAddr("RPCRT4", "UuidCreateSequential") : NULL;
+          ((HRESULT(WINAPI*)(GUID*))(func ? func : GetSysProcAddr("RPCRT4", "UuidCreate")))(&guid);
+          ((int(WINAPI*)(GUID*, TCHAR*, int))(GetSysProcAddr("OLE32", "StringFromGUID2")))(&guid, buf, 39);
+          for (UINT i = 0; sizeof(TCHAR) < 2; ++i) if (!(buf[i] = (CHAR) ((WCHAR*)buf)[i])) break; // WCHAR to TCHAR if ANSI
+          LogMessage(g_sdata.hwnd, (buf[38] = '\r', buf[39] = '\n', buf[40] = '\0', buf));
+          break;
+        }
         case IDM_TEST:
         case IDC_TEST:
         {
@@ -728,6 +774,7 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
         }
         case IDM_FIND:
         {
+          bool reuseWindow = true;
           if (!g_find.uFindReplaceMsg) g_find.uFindReplaceMsg = RegisterWindowMessage(FINDMSGSTRING);
           memset(&g_find.fr, 0, sizeof(FINDREPLACE));
           g_find.fr.lStructSize = sizeof(FINDREPLACE);
@@ -735,7 +782,8 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
           g_find.fr.Flags = FR_NOUPDOWN;
           g_find.fr.lpstrFindWhat = g_findbuf;
           g_find.fr.wFindWhatLen = COUNTOF(g_findbuf);
-          g_find.hwndFind = FindText(&g_find.fr);
+          if (!reuseWindow || !SetForegroundWindow(g_find.hwndFind))
+            g_find.hwndFind = FindText(&g_find.fr);
           return TRUE;
         }
         default:
@@ -752,7 +800,7 @@ INT_PTR CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam
       }
     }
   }
-  if (g_find.uFindReplaceMsg && msg == g_find.uFindReplaceMsg) {
+  if (msg == g_find.uFindReplaceMsg && msg) {
     LPFINDREPLACE lpfr = (LPFINDREPLACE)lParam;
     if (lpfr->Flags & FR_FINDNEXT) {
       WPARAM flags = FR_DOWN;
