@@ -5,7 +5,12 @@
 
 #include <windows.h>
 
-#define NSDFUNC __stdcall
+#define NSDFUNC WINAPI
+
+#define GetVar(vars, varlen, varid) ( (vars) + ((varid) * (varlen)) )
+#define DlgRet(hDlg, val) ( SetWindowLongPtr((hDlg), DWLP_MSGRESULT, (val)) | TRUE )
+#define StrToIntPtr nsishelper_str_to_ptr
+
 
 typedef int nsFunction;
 
