@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 2002-2018 Amir Szekely <kichik@netvision.net.il> and Contributors
+ * Copyright (C) 2002-2020 Amir Szekely <kichik@netvision.net.il> and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -430,7 +430,10 @@ void CDialogTemplate::CTrimToString(WORD id, TCHAR *str, int margins) {
   item->sWidth = short(size.cx);
   item->sHeight = short(size.cy);
 }
-#endif
+#else //! WIN32
+void CDialogTemplate::PixelsToDlgUnits(short& x, short& y) { assert(0); }
+void CDialogTemplate::DlgUnitsToPixels(short& x, short& y) { assert(0); }
+#endif //~ WIN32
 
 // Moves every item right and gives it the WS_EX_RIGHT extended style
 void CDialogTemplate::ConvertToRTL() {
