@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2018 Nullsoft and Contributors
+ * Copyright (C) 1999-2020 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -61,7 +61,7 @@
 // 5000..5999 Important generic warnings
 // 6000..6999 Script warnings
 // 7000..7499 Recovered from bad input etc. warnings
-// 7500..7999 Discouraged usage warnings (allocated top to bottom to reserve as much space as possible for more bad input codes)
+// 7500..7999 Discouraged usage warnings (allocated high to low to reserve as much space as possible for more bad input codes)
 // 8000..8999 Generic warnings
 // 9000..9999 Breaking our and/or MS guidelines warnings
 typedef enum {
@@ -107,7 +107,9 @@ typedef enum {
   DW_LICENSE_EMPTY = 7050,
   DW_ATTRIBUTE_OVERLONGSTRING = 7060,
   DW_PARSE_BADNUMBER = 7070,
+  DW_PARSE_NUMBEROUTOFSPEC = 7071,
   DW_PARSE_LNK_HK = 7075,
+  DW_GENERIC_DEPRECATED = 7998,
   DW_PARSE_REGPATHPREFIX = 7999,
   DW_INSTFILESPAGE_NOT_USED = 8000, // reserved ..8019
   DW_COMP_FINAL = 8020, // reserved ..8059
@@ -142,11 +144,12 @@ namespace MakensisAPI {
   extern const TCHAR* SigintEventNameFmt;
   extern const TCHAR* SigintEventNameLegacy;
 
-  enum notify_e {
-    NOTIFY_SCRIPT, // main nsi file(s)
+  enum datatransfer_e {
+    NOTIFY_SCRIPT, // Compiler -> Host: main nsi file(s)
     NOTIFY_WARNING,
     NOTIFY_ERROR,
-    NOTIFY_OUTPUT // generated .exe file
+    NOTIFY_OUTPUT, // Compiler -> Host: Generated .exe file
+    PROMPT_FILEPATH // [0x03006000] Compiler -> Host -> Compiler
   };
 #ifdef _WIN32
   enum sndmsg_e {
@@ -155,8 +158,14 @@ namespace MakensisAPI {
 #endif
   enum QUERYHOST_e {
     QH_OUTPUTCHARSET = 1, // [0x03000000] return (wincodepage+1) or 0 for default (This encoding is used by stdout, stderr and the notify messages)
-    QH_ENABLESTDERR // [0x03001000] return 1 to output error messages to stderr or 0 to output error messages to stdout
+    QH_ENABLESTDERR, // [0x03001000] return 1 to output error messages to stderr or 0 to output error messages to stdout
+    QH_SUPPORTEDVERSION, // [0x03006000] The host must return new highest makensis compiler version it supports
   };
+  typedef struct {
+    unsigned char Platform; // Bitness OR'ed with sizeof(TCHAR) of the compiler
+    unsigned char Reserved;
+    TCHAR Path[1];
+  } PROMPT_FILEPATH_DATA;
 }
 
 #define PAGE_CUSTOM 0
@@ -272,7 +281,14 @@ class CEXEBuild {
     NStreamLineReader* curlinereader;
 
     HWND notify_hwnd;
-    void notify(MakensisAPI::notify_e code, const TCHAR *data) const;
+    void notify(MakensisAPI::datatransfer_e code, const TCHAR *data) const;
+    #ifdef _WIN32
+    typedef bool (CALLBACK*HOSTAPIREQUESTDATAPROC)(void*cookie, HWND hWnd, UINT Msg, WPARAM wParam, LPARAM lParam);
+    #else
+    typedef bool (*HOSTAPIREQUESTDATAPROC)(void*cookie);
+    #endif
+    bool hostapi_request_data(MakensisAPI::datatransfer_e operation, UINT minver, HOSTAPIREQUESTDATAPROC proc, void*cookie, const void* input, UINT inputsize) const;
+    bool prompt_for_output_path(TCHAR*path, UINT pathcap) const;
 
   private:
     int check_write_output_errors() const;
@@ -326,6 +342,7 @@ class CEXEBuild {
     int doParse(const TCHAR *str);
     int doCommand(int which_token, LineParser &line);
     TCHAR m_templinebuf[MAX_LINELENGTH]; // Buffer used by parseScript() & doCommand(), not recursion safe!
+    int add_flag_instruction_entry(int which_token, int opcode, LineParser &line, int offset, int data = 0);
 
     int do_add_file(const TCHAR *lgss, int attrib, int recurse, int *total_files, const TCHAR 
       *name_override=0, int generatecode=1, int *data_handle=0, 
@@ -441,6 +458,7 @@ class CEXEBuild {
     void print_warnings();
     void warninghelper(DIAGCODE dc, bool fl, const TCHAR *fmt, va_list args);
     DiagState diagstate;
+    bool changed_target;
 
     /** Are we defining an uninstall version of the code?
      * @param un Use like a boolean to define whether in uninstall mode.
@@ -506,7 +524,7 @@ class CEXEBuild {
      * a PS_ERROR.  If this function call is overwriting a set user string,
      * this will return a PS_WARNING.
      */
-    int SetInnerString(int id, TCHAR *str);
+    int SetInnerString(int id, const TCHAR *str);
 
     int GenerateLangTable(LanguageTable *lt, int num_lang_tables);
     int GenerateLangTables();
@@ -550,6 +568,7 @@ class CEXEBuild {
     int DeclaredUserVar(const TCHAR *VarName);
     void VerifyDeclaredUserVarRefs(UserVarsStringList *pVarsStringList);
     bool IsIntOrUserVar(const LineParser &line, int token) const;
+    static bool IsVarPrefix(const TCHAR*s) { return *s++ == _T('$') && *s > ' '; }
 
     ConstantsStringList m_ShellConstants;
 
@@ -681,7 +700,9 @@ class CEXEBuild {
     manifest::exec_level manifest_exec_level;
     manifest::dpiaware manifest_dpiaware;
     tstring manifest_dpiawareness;
+    manifest::longpathaware manifest_lpaware;
     manifest::SupportedOSList manifest_sosl;
+    tstring manifest_maxversiontested;
 
     CResourceEditor *res_editor;
     void init_res_editor();
