@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2018 Nullsoft and Contributors
+ * Copyright (C) 1999-2020 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -176,33 +176,42 @@ static HKEY NSISCALL RegCreateScriptKey(int RootKey, LPCTSTR SubKey, REGSAM RS)
 // RegDeleteKeyEx on 32-bit Windows accepts but ignores the KEY_WOW64_xxKEY flags and always uses the 
 // one and only native key. Our RegKeyOpen will intentionally fail if a incompatible WoW64 flag is used.
 #define DRTF_ONLYIFNOSUBKEYS DELREGKEY_ONLYIFNOSUBKEYS
+#define DRTF_ONLYIFNOVALUES  DELREGKEY_ONLYIFNOVALUES
 static LONG NSISCALL DeleteRegTree(HKEY hThisKey, LPCTSTR SubKey, REGSAM SamviewAndFlags)
 {
   HKEY hKey;
-  UINT onlyifempty = SamviewAndFlags & DRTF_ONLYIFNOSUBKEYS;
+  UINT onlyifnosubkeys = SamviewAndFlags & DRTF_ONLYIFNOSUBKEYS;
+  UINT onlyifnovalues  = SamviewAndFlags & DRTF_ONLYIFNOVALUES, valuesexistcheckinsubkeys = TRUE;
   REGSAM samview = SamviewAndFlags & (KEY_WOW64_32KEY|KEY_WOW64_64KEY);
-  LONG retval = RegKeyOpen(hThisKey, SubKey, KEY_ENUMERATE_SUB_KEYS|samview, &hKey);
+  LONG retval = RegKeyOpen(hThisKey, SubKey, KEY_ENUMERATE_SUB_KEYS|KEY_QUERY_VALUE|samview, &hKey);
   if (retval == ERROR_SUCCESS)
   {
     TCHAR child[MAX_PATH+1]; // NB - don't change this to static (recursive function)
+    if (onlyifnovalues)
+    {
+      DWORD cchName = 0;
+      retval = RegEnumValue(hKey, 0, child, &cchName, NULL, NULL, NULL, NULL);
+      if (retval != ERROR_NO_MORE_ITEMS) goto notempty;
+      if (!valuesexistcheckinsubkeys) SamviewAndFlags &= ~DRTF_ONLYIFNOVALUES;
+    }
     while (RegEnumKey(hKey, 0, child, COUNTOF(child)) == ERROR_SUCCESS)
     {
-      if (onlyifempty) return (RegCloseKey(hKey), ERROR_CAN_NOT_COMPLETE);
+      if (onlyifnosubkeys) notempty: return (RegCloseKey(hKey), ERROR_CAN_NOT_COMPLETE);
       if ((retval = DeleteRegTree(hKey, child, SamviewAndFlags)) != ERROR_SUCCESS) break;
     }
     RegCloseKey(hKey);
     {
-      typedef LONG (WINAPI * RegDeleteKeyExPtr)(HKEY, LPCTSTR, REGSAM, DWORD);
-      RegDeleteKeyExPtr RDKE = (RegDeleteKeyExPtr)
-      #ifdef _WIN64
-        RegDeleteKeyEx;
-      #else
+      typedef LONG (WINAPI * RegDeleteKeyExType)(HKEY, LPCTSTR, REGSAM, DWORD);
+      RegDeleteKeyExType RDKE = (RegDeleteKeyExType)
+      #if !defined(_WIN64) || defined(_M_IA64)
         myGetProcAddress(MGA_RegDeleteKeyEx);
-      #endif
-      if (RDKE)
-        retval = RDKE(hThisKey, SubKey, samview, 0);
-      else
+      if (!RDKE)
         retval = RegDeleteKey(hThisKey, SubKey);
+      else
+      #else
+        RegDeleteKeyEx;
+      #endif
+        retval = RDKE(hThisKey, SubKey, samview, 0);
     }
   }
   return retval;
@@ -831,28 +840,18 @@ static int NSISCALL ExecuteEntry(entry *entry_)
       SetWindowLongPtr(GetHwndFromParm(0), GWLP_USERDATA, (LONG_PTR) c);
     }
     break;
-    case EW_SETBRANDINGIMAGE:
+    case EW_LOADANDSETIMAGE:
     {
       RECT r;
-      HANDLE hImage;
-      HWND hwImage=GetDlgItem(g_hwnd, parm1);
-      GetClientRect(hwImage, &r);
-      hImage=LoadImage(
-        0,
-        GetStringFromParm(0x00),
-        IMAGE_BITMAP,
-        parm2*r.right,
-        parm2*r.bottom,
-        LR_LOADFROMFILE
-      );
-      hImage = (HANDLE)SendMessage(
-        hwImage,
-        STM_SETIMAGE,
-        IMAGE_BITMAP,
-        (LPARAM)hImage
-      );
-      // delete old image
-      if (hImage) DeleteObject(hImage);
+      HANDLE hNewImage, hPrevImage;
+      HWND hCtl=(parm3 & LASIF_HWND) ? GetHwndFromParm(2) : GetDlgItem(g_hwnd, parm2);
+      UINT it=parm3 & LASIM_IMAGE, exeres=parm3 & LASIF_EXERES, fitw=(UINT)parm3 >> LASIS_FITCTLW, fith=(parm3 & LASIF_FITCTLH) != 0;
+      LPCTSTR imgid = (parm3 & LASIF_STRID) ? GetStringFromParm(0x11) : MAKEINTRESOURCE(parm1);
+      GetClientRect(hCtl, &r);
+      hNewImage=LoadImage(exeres ? g_hInstance : NULL, imgid, it, fitw*r.right, fith*r.bottom, parm3 & LASIM_LR);
+      hPrevImage=(HANDLE)SendMessage(hCtl, STM_SETIMAGE, it, (LPARAM)hNewImage);
+      if (hPrevImage && IMAGE_BITMAP == it) DeleteObject(hPrevImage); // Delete the old bitmap
+      if (parm0 >= 0) iptrtostr(var0, (INT_PTR)hNewImage); // Optional output handle
     }
     break;
     case EW_CREATEFONT:
@@ -1401,7 +1400,7 @@ static int NSISCALL ExecuteEntry(entry *entry_)
         else if (which==EW_FPUTS)
         {
           GetStringFromParm(0x21); // load string in buf2, convert it to ANSI in buf1
-          WideCharToMultiByte(CP_ACP, 0, buf2, -1, (LPSTR) buf1, NSIS_MAX_STRLEN, NULL, NULL);
+          StrWideToACP(buf2, (LPSTR) buf1, NSIS_MAX_STRLEN);
           l=lstrlenA((LPCSTR)buf1);
         }
 #endif
@@ -1715,6 +1714,32 @@ static int NSISCALL ExecuteEntry(entry *entry_)
     break;
 #endif//NSIS_CONFIG_COMPONENTPAGE
 
+    case EW_GETOSINFO:
+    {
+      //switch(parm0)
+      {
+#ifdef NSIS_SUPPORT_FNUTIL
+      //case 0:
+        {
+          TCHAR *outstr = var1;
+          IID kfid;
+          HRESULT(WINAPI*SHGKFP)(REFIID,DWORD,HANDLE,PWSTR*) = (HRESULT(WINAPI*)(REFIID,DWORD,HANDLE,PWSTR*)) myGetProcAddress(MGA_SHGetKnownFolderPath);
+          TCHAR *buf2 = GetStringFromParm(0x22), succ = FALSE;
+          if (SHGKFP && SUCCEEDED(ComIIDFromString(buf2, &kfid)))
+          {
+            PWSTR path;
+            HRESULT hr = SHGKFP(&kfid, parm3, NULL, &path);
+            if (SUCCEEDED(hr))
+              strcpyWideToT(outstr, path), CoTaskMemFree(path), ++succ;
+          }
+          if (!succ)
+            exec_error++, *outstr = _T('\0');
+        }
+        //break;
+#endif
+      }
+    }
+    break;
 #ifdef NSIS_LOCKWINDOW_SUPPORT
     case EW_LOCKWINDOW:
     {
