@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2018 Nullsoft and Contributors
+ * Copyright (C) 1999-2020 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -21,6 +21,7 @@
 #include <nsis-version.h>
 #include "tstring.h"
 #include "util.h" // RawTStrToASCII
+#include <vector>
 
 // Jim Park: The manifest must stay UTF-8.  Do not convert.
 
@@ -91,20 +92,83 @@ bool SupportedOSList::append(const TCHAR* osid)
 }
 
 
-string generate(flags featureflags, comctl comctl_selection, exec_level exec_level_selection, dpiaware dpia, const TCHAR*dpia2, SupportedOSList& sosl)
+static const TCHAR*g_appendpaths[] = { // Basic simulated XPath support
+  _T("/"),
+  _T("/assembly"),
+  _T("/assembly/dependency"),
+  _T("/assembly/dependency/dependentAssembly"),
+  _T("/assembly/compatibility/application"),
+  _T("/assembly/application/windowsSettings")
+};
+std::vector<string> g_appendstrings[COUNTOF(g_appendpaths)];
+
+static int isvalidappendpath(const TCHAR*path)
 {
-  bool default_or_empty_sosl = sosl.isdefaultlist() || !sosl.getcount();
+  for (int i = 0; i < (int) COUNTOF(g_appendpaths); ++i)
+    if (!_tcsicmp(path, g_appendpaths[i]))
+      return i;
+  return -1;
+}
+
+bool addappendstring(const TCHAR*path, const TCHAR*data)
+{
+  int i = isvalidappendpath(path);
+  if (i >= 0)
+  {
+    string str = "";
+    str += TtoCString(data);
+    g_appendstrings[i].push_back(str);
+    return true;
+  }
+  return false;
+}
+
+static bool append(string& xml, const TCHAR*path, const char*prefix = 0, const char*suffix = 0)
+{
+  bool any = false;
+  int i = isvalidappendpath(path);
+  if (i >= 0)
+    for (size_t j = 0; j < g_appendstrings[i].size(); ++j)
+    {
+      if (!any && prefix) xml += prefix, any = true;
+      xml += g_appendstrings[i][j];
+    }
+  if (any && suffix)
+    xml += suffix;
+  return any;
+}
+
+string generate(comctl comctl_selection, exec_level exec_level_selection, const SPECIFICATION&spec)
+{
+  flags featureflags = spec.Flags;
+  dpiaware dpia = spec.DPIA;
+  const TCHAR *dpia2 = spec.DPIA2;
+  longpathaware lpa = spec.lpaware;
+  SupportedOSList& sosl = *spec.pSOSL;
+  const TCHAR *mvt = spec.MaxVersionTested;
+
+  bool default_or_empty_sosl = sosl.isdefaultlist() || !sosl.getcount(), any;
   if (comctl_selection == comctl_old && exec_level_selection == exec_level_none && default_or_empty_sosl && dpiaware_notset == dpia)
     return "";
 
+  string xmltmp;
   string xml = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?><assembly xmlns=\"urn:schemas-microsoft-com:asm.v1\" manifestVersion=\"1.0\"><assemblyIdentity version=\"1.0.0.0\" processorArchitecture=\"*\" name=\"Nullsoft.NSIS.exehead\" type=\"win32\"/><description>Nullsoft Install System ";
   xml += TtoCString(NSIS_VERSION);
   xml += "</description>";
 
   if (comctl_selection == comctl_xp)
   {
-    xml += "<dependency><dependentAssembly><assemblyIdentity type=\"win32\" name=\"Microsoft.Windows.Common-Controls\" version=\"6.0.0.0\" processorArchitecture=\"*\" publicKeyToken=\"6595b64144ccf1df\" language=\"*\" /></dependentAssembly></dependency>";
+    addappendstring(_T("/assembly/dependency/dependentAssembly"), _T("<assemblyIdentity type=\"win32\" name=\"Microsoft.Windows.Common-Controls\" version=\"6.0.0.0\" processorArchitecture=\"*\" publicKeyToken=\"6595b64144ccf1df\" language=\"*\" />"));
   }
+  xmltmp = "<dependency>";
+  any = append(xmltmp, _T("/assembly/dependency"));
+  any |= append(xmltmp, _T("/assembly/dependency/dependentAssembly"), "<dependentAssembly>", "</dependentAssembly>");
+  if (any)
+  {
+    xmltmp += "</dependency>";
+    xml += xmltmp;
+  }
+
 
   if (exec_level_selection != exec_level_none)
   {
@@ -135,17 +199,25 @@ string generate(flags featureflags, comctl comctl_selection, exec_level exec_lev
     sosl.deleteall();
   }
 
+  xmltmp = "", append(xmltmp, _T("/assembly/compatibility/application"));
   int soslcount = sosl.getcount();
-  if (soslcount)
+  if (!xmltmp.empty() || soslcount || *mvt)
   {
     char buf[38+1];
     xml += "<compatibility xmlns=\"urn:schemas-microsoft-com:compatibility.v1\"><application>";
     while(soslcount--)
     {
       xml += "<supportedOS Id=\"";
-      RawTStrToASCII(sosl.get(soslcount), buf, COUNTOF(buf));
-      xml += buf, xml +=  "\"/>";
+      xml += (RawTStrToASCII(sosl.get(soslcount), buf, COUNTOF(buf)), buf);
+      xml += "\"/>";
     }
+    if (*mvt)
+    {
+      xml += "<maxVersionTested Id=\"";
+      xml += TtoCString(mvt);
+      xml += "\"/>";
+    }
+    xml += xmltmp;
     xml += "</application></compatibility>";
   }
 
@@ -165,7 +237,7 @@ string generate(flags featureflags, comctl comctl_selection, exec_level exec_lev
   if (dpiaware_notset != dpia)
   {
     xml_aws += "<dpiAware xmlns=\"http://schemas.microsoft.com/SMI/2005/WindowsSettings\">";
-    xml_aws += dpia >= dpiaware_permonitor ? "True/PM" : dpiaware_false != dpia ? "true" : "false";
+    xml_aws += dpia == dpiaware_explorer ? "Explorer" : dpia >= dpiaware_permonitor ? "True/PM" : dpiaware_false != dpia ? "true" : "false";
     xml_aws += "</dpiAware>";
   }
   if (*dpia2)
@@ -174,6 +246,13 @@ string generate(flags featureflags, comctl comctl_selection, exec_level exec_lev
     xml_aws += TtoCString(dpia2);
     xml_aws += "</dpiAwareness>";
   }
+  if (lpaware_notset != lpa)
+  {
+    xml_aws += "<longPathAware xmlns=\"http://schemas.microsoft.com/SMI/2016/WindowsSettings\">";
+    xml_aws += lpaware_false != lpa ? "true" : "false"; 
+    xml_aws += "</longPathAware>";
+  }
+  append(xml_aws, _T("/assembly/application/windowsSettings"));
   if (!xml_aws.empty())
   {
     xml += "<application xmlns=\"urn:schemas-microsoft-com:asm.v3\"><windowsSettings>";
@@ -181,7 +260,9 @@ string generate(flags featureflags, comctl comctl_selection, exec_level exec_lev
     xml += "</windowsSettings></application>";
   }
 
+  append(xml, _T("/assembly"));
   xml += "</assembly>";
+  append(xml, _T("/"));
 
   return xml;
 }
