@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2018 Nullsoft and Contributors
+ * Copyright (C) 1999-2020 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -59,13 +59,18 @@ enum
   TOK_BRANDINGTEXT,
   TOK_FILEERRORTEXT,
   TOK_INSTPROGRESSFLAGS,
+  TOK_PEADDRESOURCE,
+  TOK_PEREMOVERESOURCE,
   TOK_PEDLLCHARACTERISTICS,
   TOK_PESUBSYSVER,
   TOK_XPSTYLE,
   TOK_REQEXECLEVEL,
+  TOK_MANIFEST_APPENDCUSTOMSTRING,
   TOK_MANIFEST_DPIAWARE,
   TOK_MANIFEST_DPIAWARENESS,
+  TOK_MANIFEST_LPAWARE,
   TOK_MANIFEST_SUPPORTEDOS,
+  TOK_MANIFEST_MAXVERSIONTESTED,
   TOK_MANIFEST_DISABLEWINDOWFILTERING,
   TOK_MANIFEST_GDISCALING,
   TOK_CHANGEUI,
@@ -144,6 +149,7 @@ enum
   // section/function stuff
   TOK_SECTION,
   TOK_SECTIONEND,
+  TOK_SECTIONINSTTYPE,
   TOK_SECTIONIN,
   TOK_SECTIONGROUP,
   TOK_SECTIONGROUPEND,
@@ -275,6 +281,7 @@ enum
 #endif
   TOK_FILESEEK,
   TOK_GETFULLPATHNAME,
+  TOK_GETKNOWNFOLDERPATH,
   TOK_REBOOT,
   TOK_IFREBOOTFLAG,
   TOK_SETREBOOTFLAG,
@@ -282,6 +289,7 @@ enum
   TOK_LOGSET,
   TOK_LOGTEXT,
   TOK_SETBRANDINGIMAGE,
+  TOK_LOADANDSETIMAGE,
   TOK_SECTIONSETTEXT,
   TOK_SECTIONGETTEXT,
   TOK_SECTIONSETFLAGS,
@@ -294,8 +302,12 @@ enum
   TOK_INSTTYPEGETTEXT,
   TOK_GETCURINSTTYPE,
   TOK_SETCURINSTTYPE,
+  TOK_GETREGVIEW,
   TOK_SETREGVIEW,
+  TOK_IFALTREGVIEW,
+  TOK_GETSHELLVARCONTEXT,
   TOK_SETSHELLVARCONTEXT,
+  TOK_IFSHELLVARCONTEXTALL,
   TOK_PLUGINDIR,
   TOK_INITPLUGINSDIR,
   TOK_CREATEFONT,
@@ -305,6 +317,7 @@ enum
   TOK_IFSILENT,
   TOK_SETERRORLEVEL,
   TOK_GETERRORLEVEL,
+  TOK_IFRTLLANG,
   TOK_LOCKWINDOW,
 
   TOK__LAST,
