@@ -3,7 +3,7 @@
  * 
  * This file is a part of NSIS.
  * 
- * Copyright (C) 1999-2018 Nullsoft and Contributors
+ * Copyright (C) 1999-2020 Nullsoft and Contributors
  * 
  * Licensed under the zlib/libpng license (the "License");
  * you may not use this file except in compliance with the License.
@@ -40,6 +40,7 @@ size_t my_strftime(TCHAR *s, size_t max, const TCHAR  *fmt, const struct tm *tm)
 // Adds the bitmap in filename using resource editor re as id id.
 // If width or height are specified it will also make sure the bitmap is in that size
 int update_bitmap(CResourceEditor* re, WORD id, const TCHAR* filename, int width=0, int height=0, int maxbpp=0);
+tstring make_friendly_resource_path(const TCHAR*rt, const TCHAR*rn, LANGID rl);
 
 TCHAR* create_tempfile_path();
 tstring get_full_path(const tstring& path);
@@ -59,6 +60,7 @@ inline tstring& path_append(tstring& base, const tstring& more) { return path_ap
 inline bool IsAgnosticPathSeparator(const TCHAR c) { return _T('\\') == c || _T('/') == c; }
 bool IsWindowsPathRelative(const TCHAR *p);
 
+tstring replace_all(const TCHAR*str, const TCHAR*find, const TCHAR* replace);
 tstring lowercase(const tstring&);
 tstring get_string_prefix(const tstring& str, const tstring& separator);
 tstring get_string_suffix(const tstring& str, const tstring& separator);
